package ch.epfl.cs108;

import java.awt.image.BufferedImage;

public final class Steganographer {
    private Steganographer() {}

    // The number of bits in a char (16)
    private static int CHAR_BITS = Character.BYTES * Byte.SIZE;

    public static String extract(BufferedImage image) {
        StringBuilder b = new StringBuilder();
        int bits = 0, bitsCount = 0;
        for (int y = 0; y < image.getHeight(); ++y) {
            for (int x = 0; x < image.getWidth(); ++x) {
                int rgb = image.getRGB(x, y);
                bits = (bits << 1) | (rgb & 1);
                if (++bitsCount == CHAR_BITS) {
                    b.append((char)bits);
                    bits = bitsCount = 0;
                }
            }
        }
        return b.toString();
    }

    public static BufferedImage insert(BufferedImage image, String string) {
        BufferedImage outImage = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_RGB);
        char chr = 0;
        int chrI = 0, bitI = 0;
        for (int y = 0; y < image.getHeight(); ++y) {
            for (int x = 0; x < image.getWidth(); ++x) {
                if (bitI == 0) {
                    chr = chrI < string.length() ? string.charAt(chrI++) : 0;
                    bitI = CHAR_BITS;
                }
                int rgb = image.getRGB(x, y);
                int bit = (chr >> --bitI) & 1;
                outImage.setRGB(x, y, (rgb & ~1) | bit);
            }
        }
        return outImage;
    }
}
